var index = index_oc_ucb;
var t = 1;
var n = 100; 
var arm_n = 2;
var left = -1.5;
var right = 1.5;
var hidden = true;

var on_color = "#46bd79";
var off_color = "#bd4679";

var margin = {top: 20, right: 20, bottom: 30, left: 50},
    width = 800 - margin.left - margin.right,
    height = 300 - margin.top - margin.bottom;

var x = d3.scale.linear()
    .range([0, width]);

var y = d3.scale.linear()
    .range([height, 0]);

var xAxis = d3.svg.axis()
    .scale(x)
    .ticks(0)
    .orient("bottom");

xAxis.tickFormat("");

var yAxis = d3.svg.axis()
    .scale(y)
    .orient("left");

var line = d3.svg.line()
    .x(function(d) { return x(d.x); })
    .y(function(d) { return y(d.y); });

var area = d3.svg.area()
    .x(function(d) { return x(d.x); })
    .y0(height)
    .y1(function(d) { return y(d.y); });

var svg = d3.select("#graph").append("svg")
    .attr("width", width + margin.left + margin.right)
    .attr("height", height + margin.top + margin.bottom)
  .append("g")
    .attr("transform", "translate(" + margin.left + "," + margin.top + ")");


function Arm(mu, id, color) {
  d3.select("#buttons").append("div")
      .attr("class", "menu")
      .append("div")
      .attr("class", "round")
      .attr("id", id)
      .attr("style", "background-color:" + color)
      .text(1);

  this.color = color;
  this.id = id;
  this.T = 0;
  this.gen = new Ziggurat();
  this.mu = mu;
  this.sum = 0.0;
  this.hat_mu = this.sum;

  this.randomise = function() {
    this.samples = [];
    for (var i = 0;i != n;++i) {
      if (Math.random() < this.mu) {
        this.samples.push(1.0 - this.mu); 
      }else {
        this.samples.push(-this.mu);
      }
    }
  }

  this.randomise();

  this.pull = function() {
    this.sum+=this.samples[this.T] + this.mu;
    this.T += 1;
    this.hat_mu = this.sum / this.T;
  }

  this.normal_pdf = function(x, mu, v) {
    return Math.exp(-(mu - x) * (mu - x) / (2.0 * v));
  }

  this.gen_data = function(idx) {
    var pdf_data = [];
    var area_data = [];
  
    var samples = 500;
    var step = (right - left) / samples;
    for (var k = 0;k != samples;++k) {
      var x1 = k * step + left;
      var y1 = this.normal_pdf(x1, this.hat_mu, 1.0 / this.T);

      pdf_data.push({x : x1, y : y1});
      if (x1 < idx && x1 >=this.hat_mu) {
        area_data.push({x : x1, y: y1});
      }
    }
    return {"line" : pdf_data, "area" : area_data};
  }
}

function update() {
  d3.select("#timestep").text(t);
  d3.selectAll(".line").remove();
  d3.selectAll(".area").remove();
  d3.selectAll(".true_line").remove();
  d3.selectAll(".box").remove();
  d3.selectAll(".text").remove();

  var max_mu = -1000;
  var regret = 0.0;

  var idx = 0;
  arms.forEach(function(arm) {
    if (arm.mu > max_mu) {
      max_mu = arm.mu;
    }
    data = arm.gen_data(index(arm.hat_mu, n, t, arm.T));
    d3.select("#" + arm.id).text(arm.T);

    
    svg.append("path")
      .datum([{x:idx + 0.25,y:arm.hat_mu},{x:idx - 0.25,y:arm.hat_mu}])
      //.datum([{x:idx + 0.25,y:0.0},{x:idx + 0.25,y:arm.hat_mu},{x:idx - 0.25,y:arm.hat_mu},{x:idx - 0.25, y:0.0}])
      .attr("class", "box")
      .attr("style", "stroke:" + arm.color)
      .attr("fill", "none")
      .attr("d", line)
    svg.append("text")
      .attr("transform", "translate(" + x(idx) + "," + y(arm.hat_mu) + ")")
      .attr("class", "text")
      .attr("text-anchor", "middle")
      .attr("y", 30)
      .text(Math.round(arm.hat_mu*100)/100.0)
    if (!hidden) {
      var upper_conf = index(arm.hat_mu, n, t, arm.T);
      svg.append("path")
        .datum([{x:idx,y:arm.hat_mu},{x:idx,y:upper_conf}])
        .attr("class", "box")
        .attr("style", "stroke:" + arm.color)
        .attr("d", line)
      svg.append("path")
        .datum([{x:idx - 0.125,y:upper_conf},{x:idx+0.125,y:upper_conf}])
        .attr("class", "box")
        .attr("style", "stroke:" + arm.color)
        .attr("d", line)
      svg.append("path")
        .datum([{x:-0.5,y:arm.mu},{x:1.5,y:arm.mu}])
        .attr("class", "true_line")
        .attr("style", "stroke:" + arm.color)
        .attr("d", line)
    }
/*
    svg.append("path")
      .datum(data["area"])
      .attr("class", "area")
      .attr("style", "fill:" + arm.color)
      .style("fill-opacity", opacity) 
      .attr("d", area);*/
    idx+=1;
  });
  var regret = 0.0;
  var reward = 0.0;
  arms.forEach(function(arm) {
    regret+= arm.T * (max_mu - arm.mu);
    reward+= arm.T * arm.hat_mu;
  });
  if (!hidden) {
    d3.select("#regret").text(Math.round(regret*1000)/1000);
  }
  d3.select("#reward").text(Math.round(reward));
}

function show() {
  d3.select("#regret").style("color", "black");
  hidden = false;
  update();
}

function hide() {
  hidden = true;
  d3.select("#regret").style("color", "white");
}

function get_arms() {
  var a = [];
  var colors = ["#4697bd", "#bd7946", "#79bd46", "#46bd79"];
  for (var i = 0;i != arm_n;i++) {
//    a.push(new Arm(Math.random() - 0.5, "arm" + i.toString(), colors[i]));
    a.push(new Arm(Math.random() * 0.6 + 0.2, "arm" + i.toString(), colors[i]));
  }
  d3.select("#buttons").append("div").attr("class", "clear");
  return a;
}

function random() {
  d3.select("#buttons").html("");
  arms = get_arms();
 

  for (var i = 0;i != arms.length;i++) {
    (function() {
      var j = i;
      d3.select("#" + arms[i].id).on("click", function() {
        arms[j].pull();
        t+=1;
        update();
      });
    })();
  }
  reset();
}

function randomise_data() {
  arms.forEach(function(arm) {
    arm.randomise();
  });
  reset();
}

function reset() {
  arms.forEach(function(arm) {
    arm.T = 0;
    arm.sum = 0.0;
    arm.pull();
    t = arms.length;
  });
  update();
}

x.domain([-0.5,1.5]);
y.domain([0,1]);

random();

d3.selectAll("span.button").style("background", off_color);
d3.select("#ocucb").style("background", on_color);
index = index_oc_ucb;


function run() {
  t+=1;
  var max_i = -1;
  var max_idx = -10000;
  for (var i = 0;i != arms.length;i++) {
    var idx = index(arms[i].hat_mu, n, t, arms[i].T);
    if (idx > max_idx) {
      max_i = i;
      max_idx = idx;
    }
  }
  arms[max_i].pull();
  update();
  if (t == n) {
    return true;
  }
  return false;
}

//d3.select("#horizon").text(n);

d3.select("#run").on("click", function() {
  reset();
  d3.timer(run, 0, 100);
});

d3.select("#reset").on("click", function() {
  reset();
});

d3.select("#reveal").on("click", function() {
  show();
});

d3.select("#random").on("click", function() {
  random();
});

d3.select("#display").on("change", function() {
  var type = this.checked;
  if (type == true) {
    show();
  }else {
    hide();
    update();
  }
});

d3.select("#algorithm").on("change", function() {
  var alg = this.options[this.selectedIndex].value;
  if (alg == 'ucb') {
    index = index_ucb;
  }else if (alg == 'ocucb') {
    index = index_oc_ucb;
  }
});


d3.select("#arms").on("change", function() {
  arm_n = parseInt(this.options[this.selectedIndex].text);
  random();
});

d3.select("#horizon").on("change", function() {
  n = parseInt(this.options[this.selectedIndex].text);
  randomise_data();
});

d3.select("#randomise_data").on("click", function() {
  randomise_data();
});


svg.append("g")
    .attr("class", "x axis")
    .attr("transform", "translate(0," + height + ")")
    .call(xAxis);

svg.append("g")
    .attr("class", "y axis")
    .call(yAxis);

